#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sgl+.h>

#define tmax 1000.
#define dt   0.05
#define x0   0.5
#define y0   0.5
#define r    0.02
#define imax 10
#define M    20
#define N    5

struct Blick {
  int id;
  double x;
  double y;
  int flag;
};

typedef Blick BLICK;
int  check( float x, float y, float vx, float vy, BLICK * pblick );
void DrawBlick( BLICK* pblick );
void check_wall( float *x,  float* y, float *vx, float* vy, 
		 BLICK blickstack[][N] );

int red, blue, green, yellow;
int color[N];

int main(int argc, char *argv[]){
  
  int i,j,k;
  float x,y,t,dx,dy,xb,yb;
  float vx =  0.3204883458;
  float vy = -0.1083472131;
  float halfwidth =0.5/M;
  float halfheight=0.05/2.0;
  
  sgl_winsize(500,500);
  sgl_init();

  color[0]=red    = sgl_colordef("red");
  color[1]=blue   = sgl_colordef("blue");
  color[2]=green  = sgl_colordef("green");
  color[3]=yellow = sgl_colordef("yellow");
  color[4]=yellow = sgl_colordef("skyblue");

  BLICK blickstack[M][N];
  for(k=0,i=0;i<M;i++){
    for(j=0;j<N;j++){
      blickstack[i][j].id = j;
      blickstack[i][j].x  = halfwidth+halfwidth*2*i;
      blickstack[i][j].y  = 0.9-halfheight*2*j;
      blickstack[i][j].flag = 1;
    }
  }

  x=x0;
  y=y0;
  xb=x0;
  yb=y0;
  dx=vx*dt;
  dy=vy*dt;

  for(t=0;t<tmax;t+=dt){
    
    x+=vx*dt;
    y+=vy*dt;
    if( x<=0. || x>=1. ){ vx = -vx; }
    if( y<=0. || y>=1. ){ vy = -vy; }

    check_wall( &x, &y, &vx, &vy, blickstack );

    printf("( %f , %f ) \n",x,y);
    
    sgl_disc(x,y,r);
    sgl_rectangle(0.0,0.005,0.998,0.99);
    for(i=0; i<M; i++ ){ for(j=0; j<N; j++ ){
      DrawBlick( &(blickstack[i][j]) );
    }}
    sgl_display();
    
    xb=x;
    yb=y;
  }

  i=getchar();

}

void DrawBlick( BLICK* pblick )
{
  if( pblick -> flag == 0 ){ return; }
  float x = pblick -> x;
  float y = pblick -> y;
  int   id= pblick -> id; 
  float halfwidth =0.5/M;
  float halfheight=0.05/2.0;
  
  sgl_color(color[id]);
  sgl_rectarea(x-halfwidth,y-halfheight,
	       halfwidth*2.0,halfheight*2.0);
  sgl_color(SGL_WHITE);
  sgl_rectangle(x-halfwidth,y-halfheight,
		halfwidth*2.0,halfheight*2.0);

}
void check_wall( float* px,  float* py, float* pvx, float* pvy, 
		 BLICK blickstack[][N] )
{
  int i,j,flag;
  for(i=0; i<M; i++ ){ 
    for(j=0; j<N; j++ ){
      if( blickstack[i][j].flag == 0 ){ continue; }
      int status = check( *px, *py, *pvx, *pvy, &(blickstack[i][j]) );
      if( status ){ 
	blickstack[i][j].flag = 0;
	putchar(7);
	switch( status ){
	case 1 : 
	  *pvx = - *pvx;
	  return;
	  break;
	case 2 :
	  *pvy = - *pvy;
	  return;
	  break;
	case 3 :
	  *pvx = - *pvx;
	  *pvy = - *pvy;
	  //fprintf(stderr,"incredible!");
	  //exit(1);
	  break; 
	}
      }
    }
  }
}
int check( float x, float y, float vx, float vy, BLICK * pblick )
{

  /*
                       (xo,y1)     case4     (x1,y1)
                          *---------------------*
                          |  (x,y)              |
          (xprev,yprev)   |   *                 |
		*       case1                  case2
                          |        case3        |
                          *---------------------*
                       (xo, yo)              (x1,yo)
		       */

  float halfwidth =0.5/M;
  float halfheight=0.05/2;
  float xprev = x - vx*dt;
  float yprev = y - vy*dt;
  
  float xo = pblick -> x - halfwidth;
  float yo = pblick -> y - halfheight;
  float x1 = xo + halfwidth *2.0;
  float y1 = yo + halfheight*2.0;

  //printf( "( x - xo )*( x - x1 )=%f\n",( x - xo )*( x - x1 ));
  //printf( "( y - yo )*( y - y1 )=%f\n",( y - yo )*( y - y1 ));

  if( ( x - xo )*( x - x1 ) < 0 && ( y - yo )*( y - y1 ) < 0 ){

    if(       (x-xo)*(xprev-xo) <  0 ){ 
      return(1);  
    }else if( (x-x1)*(xprev-x1) <  0 ){ 
      return(1); 
    }else if( (y-yo)*(yprev-yo) <= 0 ){ 
      return(2); 
    }else if( (y-y1)*(yprev-y1) <= 0 ){ 
      return(2); 
    }else{ return(3); }

  }else{
    return( 0 );
  }

}
