
/*
  << Compile Library >>

   C++ version 


  (1) g++ -O3 -I/usr/include/X11 -c libsgl+.C -D__STDC__

  (2) ar rv libsgl++.a libsgl+.o

  (3) ranlib libsgl++.a

   libsgl++.a 

*/
/************************************************************************/
/*									*/
/* LIBSGL++.C   ver.14.0						*/
/*									*/
/************************************************************************/
/*									*/
/*	LIBSGL.C				(C) 9/94 Ch. Muenkel	*/
/*	Main part of SGL library -- basic subroutines			*/
/*									*/
/************************************************************************/
/*									*/
/*	Simple Graphics Library						*/
/*	An easy to use interface to the X11 window system		*/
/*									*/
/************************************************************************/
/*									*/
/*	Author:	Christian Muenkel					*/
/*		Present address:					*/
/*		muenkel@carbo.tphys.uni-heidelberg.de			*/
/*		Christian Muenkel					*/
/*		Institut fuer Theoretische Physik			*/
/*		Philosophenweg 19					*/
/*		69120 Heidelberg - Germany				*/
/*									*/
/************************************************************************/
/* version      date            comment                                 */
/*                                                                      */
/* V1.0		 1/94		first implementation 			*/
/*		 9/94		final tests                             */
/*                                                                      */
/************************************************************************/

#define		__SGL_MAIN
#include	"sglglob+.h"
#undef		__SGL_MAIN

#ifdef	__STDC__
static	void	sgl_create_colmap(void);
static	void	sgl_priv_init(int num_win);
#else
static void	sgl_create_colmap();
static	void	sgl_priv_init();
#endif

#define DEBUG /* Added by T.Yamaue */

/************************************************************************/
/************************************************************************/
/*									*/
/* INITIALIZATIONS ...							*/
/*									*/
/************************************************************************/
/************************************************************************/


/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_init(void)						*/
/*		initialize X11 graphics					*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void	sgl_init(void)
#else
  void	sgl_init()
#endif
{
  sgl_priv_init(1);
}

#ifdef	__STDC__
void	sgl_initm(int	num_win)
#else
  void	sgl_initm(num_win)
  int	num_win;
#endif
{
  sgl_priv_init(num_win);
}

#ifdef	__STDC__
static void	sgl_priv_init(int	num_win)
#else
  static void	sgl_priv_init(num_win)
  int	num_win;
#endif
{
  int		i,mask;
  XVisualInfo	visinf;
  XSetWindowAttributes attr;
  unsigned long gc_valmask;
  XGCValues	gcval;
  char		str[1024];

  if(flag_initialized == True)
    return;

  GD_num = num_win;
  if(GD_num <= 0)			/* nothing to initialize	*/
    { FatalError("SGL_INIT: argument must be positive.\n");
    }
  else if(GD_num > MAX_X11GD_STRUCT)	/* more than we can do		*/
    { FatalError("SGL_INIT: Too many windows.\n");
    }
	
  win_x_min = 0.0;
  win_x_max = 1.0;
  win_y_min = 0.0;
  win_y_max = 1.0;
  win_idx = 1.0 / (win_x_max - win_x_min);
  win_idy = 1.0 / (win_y_max - win_y_min);

  /***** allocate memory ************************************************/
  GD_act = 0;
  for (i = 0; i < GD_num; i++)
    { X11GD[i]=(struct X11GlobalData *)malloc(sizeof(struct X11GlobalData));
    if(X11GD[i] == NULL)
      FatalError("SGL_INIT: Can't allocate X11 global data structure.\n");
    X11GD[i]->win_sx = win_sx;
    X11GD[i]->win_sy = win_sy;
    X11GD[i]->win_x_min = (int)win_x_min;
    X11GD[i]->win_y_min = (int)win_y_min;
    X11GD[i]->win_x_max = (int)win_x_max;
    X11GD[i]->win_y_max = (int)win_y_max;
    X11GD[i]->win_idx   = (int)win_idx;
    X11GD[i]->win_idy   = (int)win_idy;
    }

  /***** allocate color table ***********************************************/
  color_pixel = (unsigned long *) calloc(COLOR_MAX,sizeof(unsigned long));
  if(color_pixel == NULL)
    FatalError("SGL_INIT: Can't allocate color data structure.\n");
  // Added by T.Yamaue 98/12/04
  color_rgb = (XColor *) calloc(COLOR_MAX,sizeof(XColor));
  if(color_rgb == NULL)
    FatalError("SGL_INIT: Can't allocate color rgb data structure.\n");

  /***** Open X window display ******************************************/
  X11GD[0]->disp_name = NULL;
  X11GD[0]->disp = XOpenDisplay( X11GD[0]->disp_name );
  if( ! X11GD[0]->disp )
    { sprintf(str,"Can't open display %s .\n",
	      XDisplayName(X11GD[0]->disp_name));
    FatalError(str);
    }
  X11GD[0]->scrnn = DefaultScreen( X11GD[0]->disp);
  X11GD[0]->scrn  = ScreenOfDisplay( X11GD[0]->disp, X11GD[0]->scrnn);
  i = 1;
  while (i < GD_num)
    { if(X11GD[0]->disp_name == NULL)
      X11GD[i]->disp_name = X11GD[0]->disp_name;
    else
      strcpy(X11GD[i]->disp_name,X11GD[0]->disp_name);
    X11GD[i]->disp  = X11GD[0]->disp;
    X11GD[i]->scrnn = X11GD[0]->scrnn;
    X11GD[i]->scrn  = X11GD[0]->scrn ;
    i++;
    }

  /***** Detect color model used by display ******************************/
  /***** first, check for best: TrueColor(24bits) display ************************/
  if( XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			24, TrueColor, &visinf))
    { X11GD[0]->vistype = VISTYPE_TrueColor;
    X11GD[0]->vis = visinf.visual;
    X11GD[0]->depth = visinf.depth;
    sgl_create_colmap();			/* create colormap	*/
    mask = 0;
    attr.colormap = X11GD[0]->colmap;
    mask |= CWColormap;
    attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
    mask |= CWBorderPixel;
    attr.backing_store = Always;
    mask |= CWBackingStore;
    X11GD[0]->win = XCreateWindow(X11GD[0]->disp,
				  RootWindow(X11GD[0]->disp,X11GD[0]->scrnn),
				  0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
				  X11GD[0]->vis, mask, &attr);
    XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL TrueColor");
    X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
				 win_sx,win_sy,visinf.depth);
    if(!X11GD[0]->pm) 
      FatalError("SGL_INIT: Couldn't create TrueColor X pixmap.");
    gc_valmask = 0;
    X11GD[0]->gc = XCreateGC( X11GD[0]->disp, X11GD[0]->win,
			      gc_valmask, &gcval);
    }
  /***** first, check for best: TrueColor(16bits) display  Added by T.Yamaue ***/
  else if( XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			     16, TrueColor, &visinf))
    { X11GD[0]->vistype = VISTYPE_TrueColor;
    X11GD[0]->vis = visinf.visual;
    X11GD[0]->depth = visinf.depth;
    sgl_create_colmap();			/* create colormap	*/
    mask = 0;
    attr.colormap = X11GD[0]->colmap;
    mask |= CWColormap;
    attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
    mask |= CWBorderPixel;
    attr.backing_store = Always;
    mask |= CWBackingStore;
    X11GD[0]->win = XCreateWindow(X11GD[0]->disp,
				  RootWindow(X11GD[0]->disp,X11GD[0]->scrnn),
				  0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
				  X11GD[0]->vis, mask, &attr);
    XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL TrueColor");
    X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
				 win_sx,win_sy,visinf.depth);
    if(!X11GD[0]->pm) 
      FatalError("SGL_INIT: Couldn't create TrueColor X pixmap.");
    gc_valmask = 0;
    X11GD[0]->gc = XCreateGC( X11GD[0]->disp, X11GD[0]->win,
			      gc_valmask, &gcval);
    }
  /***** second, check for PseudoColor display ************************/
  else if(XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			    8,PseudoColor,&visinf))
    { X11GD[0]->vistype = VISTYPE_PseudoColor;
    X11GD[0]->vis = visinf.visual;
    X11GD[0]->depth = visinf.depth;
    sgl_create_colmap();			/* create colormap	*/
    mask = 0;
    attr.colormap = X11GD[0]->colmap;
    mask |= CWColormap;
    attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
    mask |= CWBorderPixel;
    attr.backing_store = Always;
    mask |= CWBackingStore;
    X11GD[0]->win = XCreateWindow(X11GD[0]->disp, 
				  RootWindow(X11GD[0]->disp, X11GD[0]->scrnn),
				  0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
				  X11GD[0]->vis, mask, &attr);
    XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL PseudoColor");
    X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
				 win_sx,win_sy,visinf.depth);
    if(!X11GD[0]->pm) 
      FatalError("SGL_INIT: Couldn't create PseudoColor X pixmap.");
    gc_valmask = 0;
    X11GD[0]->gc = XCreateGC(X11GD[0]->disp, X11GD[0]->win,
			     gc_valmask, &gcval);
    }
  /***** third, check for StaticGray, GrayScale display ****************/
  else if(XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			    1,StaticGray,&visinf)  ||
	  XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			    1,GrayScale,&visinf))
    { if(XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			   1,StaticGray,&visinf))
      X11GD[0]->vistype = VISTYPE_StaticGray;
    else if(XMatchVisualInfo( X11GD[0]->disp, X11GD[0]->scrnn,
			      1,GrayScale,&visinf))
      X11GD[0]->vistype = VISTYPE_GrayScale;
    X11GD[0]->depth = visinf.depth;
    mask = 0;
    attr.colormap = X11GD[0]->colmap;
    mask |= CWColormap;
    attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
    mask |= CWBorderPixel;
    attr.backing_store = Always;
    mask |= CWBackingStore;
    X11GD[0]->win = XCreateWindow(X11GD[0]->disp,
				  RootWindow(X11GD[0]->disp, X11GD[0]->scrnn),
				  0,0, win_sx,win_sy, 1, visinf.depth, InputOutput,
				  X11GD[0]->vis, mask, &attr);
    XStoreName(X11GD[0]->disp, X11GD[0]->win, "SGL Gray/BW");
    X11GD[0]->pm = XCreatePixmap(X11GD[0]->disp,X11GD[0]->win,
				 win_sx,win_sy,visinf.depth);
    if(!X11GD[0]->pm) 
      FatalError("SGL_INIT: Couldn't create StaticGray/GrayScale X pixmap.");
    gc_valmask = 0;
    X11GD[0]->gc = XCreateGC(X11GD[0]->disp, X11GD[0]->win,
			     gc_valmask, &gcval);
    color_pixel[0]= BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
    color_pixel[1]= WhitePixel(X11GD[0]->disp,X11GD[0]->scrnn);
    color_stack_top = 2;
    }
  else
    { FatalError("SGL_INIT: Sorry, this type of Visual can't be used.\n");
    }

  i = 1;
  while (i < GD_num)			/* initialize second ... window	*/
    { X11GD[i]->vistype = X11GD[0]->vistype;
    X11GD[i]->vis     = X11GD[0]->vis;
    X11GD[i]->depth   = X11GD[0]->depth;
    X11GD[i]->colmap  = X11GD[0]->colmap;
    mask = 0;
    attr.colormap = X11GD[0]->colmap;
    mask |= CWColormap;
    attr.border_pixel = BlackPixel(X11GD[0]->disp,X11GD[0]->scrnn);
    mask |= CWBorderPixel;
    attr.backing_store = Always;
    mask |= CWBackingStore;
    X11GD[i]->win = XCreateWindow(X11GD[0]->disp, 
				  RootWindow(X11GD[0]->disp, X11GD[0]->scrnn),
				  0,0, win_sx,win_sy, 1, X11GD[0]->depth, InputOutput,
				  X11GD[0]->vis, mask, &attr);
    sprintf(str,"Window %d",i+1);
    XStoreName(X11GD[i]->disp, X11GD[i]->win, str);
    X11GD[i]->pm = XCreatePixmap(X11GD[i]->disp,X11GD[i]->win,
				 win_sx,win_sy,X11GD[i]->depth);
    if(!X11GD[i]->pm) 
      FatalError("SGL_INIT: Couldn't create PseudoColor X pixmap.");
    gc_valmask = 0;
    X11GD[i]->gc = XCreateGC( X11GD[i]->disp, X11GD[i]->win,
			      gc_valmask, &gcval);
    i++;
    }

  for (i = 0; i < GD_num; i++)
    { XMapRaised(X11GD[i]->disp, X11GD[i]->win);
    XMoveWindow(X11GD[i]->disp, X11GD[i]->win,win_px,win_py);
    XSetWindowColormap( X11GD[i]->disp, X11GD[i]->win,
			X11GD[i]->colmap);
    }

  if(debugging == True) 
    { switch(X11GD[0]->vistype)
      { case VISTYPE_GrayScale:
	  fprintf(stderr,"SGL: display type is GrayScale.\n");
	  break;
      case VISTYPE_StaticGray:
	fprintf(stderr,"SGL: display type is StaticGray.\n");
	break;
      case VISTYPE_PseudoColor:
	fprintf(stderr,"SGL: display type is PseudoColor.\n");
	break;
      case VISTYPE_TrueColor:
	fprintf(stderr,"SGL: display type is TrueColor.\n");
	break;
      default:
	fprintf(stderr,"SGL: display type is unknown.\n");
	break;
      }
    }

  for (i = 0; i < GD_num; i++)
    { XSync(X11GD[i]->disp, X11GD[i]->win);
    XSetWindowColormap(X11GD[i]->disp, X11GD[i]->win, X11GD[i]->colmap);

    XSetBackground( X11GD[i]->disp, X11GD[i]->gc, color_pixel[0]);
    XSetForeground( X11GD[i]->disp, X11GD[i]->gc, color_pixel[0]);
    XFillRectangle( X11GD[i]->disp, X11GD[i]->pm, X11GD[i]->gc,
		    0,0,win_sx,win_sy);
    XDrawLine( X11GD[i]->disp, X11GD[i]->pm, X11GD[i]->gc,
	       0,0,win_sx,win_sy);
    XCopyArea( X11GD[i]->disp, X11GD[i]->pm, X11GD[i]->win,
	       X11GD[i]->gc, 0,0,win_sx,win_sy,0,0);
    XSetForeground( X11GD[i]->disp, X11GD[i]->gc, color_pixel[1]);
    XFlush(X11GD[i]->disp);
    XSync(X11GD[i]->disp, X11GD[i]->win);
    }

  color_background = 0;
  color_foreground = 1;
  flag_initialized = True;

}
/************************************************************************/
/************************************************************************/
/*									*/
/* DRAWING FUNCTIONS						        */
/*									*/
/************************************************************************/
/************************************************************************/

/************************************************************************/
/************************************************************************/
/*									                                                    */
/* void	sgl_line(x1,y1,x2,y2)                                           */
/*	draw a line from (x1,y1) to (x2,y2) in viewport coord.              */
/*									                                                    */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_line(float x1, float y1, float x2, float y2)
#else
  void	sgl_line(x1,y1,x2,y2)
  float	x1,y1,x2,y2;
#endif
{
  int	ix1,iy1,ix2,iy2;

  ix1 = (int)( ((x1 - win_x_min) * win_idx) * win_sx );
  ix2 = (int)( ((x2 - win_x_min) * win_idx) * win_sx );
  iy1 = (int)( win_sy - ((y1 - win_y_min) * win_idy) * win_sy );
  iy2 = (int)( win_sy - ((y2 - win_y_min) * win_idy) * win_sy );

  if(flag_initialized == True)
    XDrawLine(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	      ix1,iy1,ix2,iy2);
}

/************************************************************************/
/************************************************************************/
/*									                                                    */
/*  void	sgl_circle(x,y,r)	                                            */
/*	draw a circle at (x,y) with radius r in viewport coord              */
/*									                                                    */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_circle(float x, float y, float r)
#else
  void	sgl_circle(x,y,r)
  float	x,y,r;
#endif
{
  int	ix,iy,ir;

  ix = (int)( (((x-r) - win_x_min) * win_idx) * win_sx );
  iy = (int)( win_sy - (((y+r) - win_y_min) * win_idy) * win_sy );
  ir = (int)( (r * win_idx) * win_sx * 2 );

  if(flag_initialized == True)
    XDrawArc(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	     ix,iy,ir,ir,0,64*360);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_disc(x,y,r)						*/
/*  draw a filled circle at (x,y) with radius r in viewport		*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_disc(float x, float y, float r)
#else
  void	sgl_disc(x,y,r)
  float	x,y,r;
#endif
{
  int	ix,iy,ir;

  ix = (int)( (((x-r) - win_x_min) * win_idx) * win_sx );
  iy = (int)( win_sy - (((y+r) - win_y_min) * win_idy) * win_sy );
  ir = (int)( (r * win_idx) * win_sx * 2 );

  if(flag_initialized == True)
    XFillArc(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
	     ix,iy,ir,ir,0,64*360);
}

/************************************************************************/
/************************************************************************/
/*     							                */
/*  void	sgl_rectangle(x,y,dx,dy)				*/
/*		draw a rectangle; (x,y) is the lower-right corner	*/
/*		(dx,dy) are the width/ height of the rectangle		*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_rectangle(float x, float y, float dx, float dy)
#else
  void	sgl_rectangle(x,y,dx,dy)
  float	x,y,dx,dy;
#endif
{
  int	ix,iy,idx,idy;

  ix  = (int)( ((x - win_x_min) * win_idx) * win_sx );
  idx = (int)( (dx * win_idx) * win_sx );
  iy  = (int)( win_sy - (((y+dy) - win_y_min) * win_idy) * win_sy );
  idy = (int)( (dy * win_idy) * win_sy );

  if(flag_initialized == True)
    XDrawRectangle(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
		   ix,iy,idx,idy);
}

/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_rectarea(x,y,dx,dy)					*/
/*		draw a filled rectangle; (x,y) is the lower-right       */
/*		corner, (dx,dy) are the width/ height of the rectangle	*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_rectarea(float x, float y, float dx, float dy)
#else
  void	sgl_rectarea(x,y,dx,dy)
  float	x,y,dx,dy;
#endif
{
  int	ix,iy,idx,idy;

  ix  = (int)( ((x - win_x_min) * win_idx) * win_sx );
  idx = (int)( (dx * win_idx) * win_sx );
  iy  = (int)( win_sy - (((y+dy) - win_y_min) * win_idy) * win_sy );
  idy = (int)( (dy * win_idy) * win_sy );

  if(flag_initialized == True)
    XFillRectangle(X11GD[GD_act]->disp,X11GD[GD_act]->pm,X11GD[GD_act]->gc,
		   ix,iy,idx,idy);
}
/************************************************************************/
/*									                                                    */
/* Following Functions are added by T. Taniguchi                        */
/*									                                                    */
/************************************************************************/
/************************************************************************/
/*									                                                    */
/* void	sgl_arrow(x1,y1,v1,v2)                                          */
/*	draw an arrow from (x1,y1) to (x1+vx,y1+vy) in viewport coord.      */
/*									                                                    */
/************************************************************************/
float angle( float vx, float vy )
{
	float ang;
	if( vx == 0.0 && vy == 0.0 ){ return 0.0; }

	float v  = sqrt( vx*vx + vy*vy );	
	float ex = vx/v, ey = vy/v;
	
	if( ey > 0.0 ){
		ang = acos( ex );
	}else{
		if( ex > 0.0 ){
			ang = asin( ex );
		}else{
			ang = 2.0*M_PI - acos( ex );
		}
	}
	return ang;
}
/************************************************************************/
#ifdef  __STDC__ 
void sgl_arrow( float x1, float y1, float vx, float vy, 
								float size, int flag )
#else
void sgl_arrow(x1,y1,vx,vy, size, flag)
float	x1,y1,vx,vy, size; int flag;
#endif
{
	float x2 = x1 + vx;
	float y2 = y1 + vy;
	float v  = sqrt( vx*vx + vy*vy );
  int	ix1,iy1,ix2,iy2;

	if( vx == 0.0  && vy == 0.0 ){ return; }

	float theta = angle( vx, vy );

	float sin_th = 0.5;
	float cos_th = sqrt(3.0)*0.5;

  ix1 = (int)( ((x1 - win_x_min) * win_idx) * win_sx );
  ix2 = (int)( ((x2 - win_x_min) * win_idx) * win_sx );
  iy1 = (int)( win_sy - ((y1 - win_y_min) * win_idy) * win_sy );
  iy2 = (int)( win_sy - ((y2 - win_y_min) * win_idy) * win_sy );

	//----------------------------
	// Hat at Tip //
	//----------------------------
	vx *= size; 	vy *= size;
	if( flag != 0 ){ vx /= v; vy /= v; }

	float vxa =    cos_th * (-vx) + sin_th * (-vy);
	float	vya	=  - sin_th * (-vx) + cos_th * (-vy);

	float vxb =   cos_th * (-vx) - sin_th * (-vy);
	float	vyb	=   sin_th * (-vx) + cos_th * (-vy);

	float x3 = x2 + vxa;
	float y3 = y2 + vya;
	float x4 = x2 + vxb;
	float y4 = y2 + vyb;

  int	ix3,iy3,ix4,iy4;

  ix3 = (int)( ((x3 - win_x_min) * win_idx) * win_sx );
  ix4 = (int)( ((x4 - win_x_min) * win_idx) * win_sx );
  iy3 = (int)( win_sy - ((y3 - win_y_min) * win_idy) * win_sy );
  iy4 = (int)( win_sy - ((y4 - win_y_min) * win_idy) * win_sy );

  if(flag_initialized == True){
    XDrawLine( X11GD[GD_act]->disp,
							 X11GD[GD_act]->pm,
							 X11GD[GD_act]->gc,
							 ix1,iy1,ix2,iy2 );

    XDrawLine( X11GD[GD_act]->disp,
							 X11GD[GD_act]->pm,
							 X11GD[GD_act]->gc,
							 ix2,iy2,ix3,iy3 );

    XDrawLine( X11GD[GD_act]->disp,
							 X11GD[GD_act]->pm,
							 X11GD[GD_act]->gc,
							 ix2,iy2,ix4,iy4 );
	}
}
/************************************************************************/
/*									                                                    */
/*  void	sgl_text(x,y,string)                                          */
/*									                                                    */
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_text(float x,float y,char string[])
#else
  void	sgl_text(x, y, string[])
  float x,y; char string[];
#endif
{
  int ix, iy;

  ix = (int)( ((x - win_x_min) * win_idx) * win_sx );
  iy = (int)( win_sy - ((y - win_y_min) * win_idy) * win_sy );

  if(flag_initialized == True)
    XSetFont(X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
	     XLoadFont(X11GD[GD_act]->disp,"clB8x14"));

  /*               XLoadFont(X11GD[GD_act]->disp,"6x10"));*/

  /* Location of Fonts : /usr/lib/X11/fonts */
  XDrawString(X11GD[GD_act]->disp,
	      X11GD[GD_act]->pm,
	      X11GD[GD_act]->gc,
	      ix,iy,string,5);
}
/************************************************************************/
/*                                     					*/
/*  void	sgl_text2(x,y,string1,Alphabet)                	        */
/*	                                   				*/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_text2(float x,float y,char *string1,char *string2)
#else
  void	sgl_text2(x, y, string1[], string2[])
  float x,y; char string1[], char string2[];
#endif
{
  int	ix, iy;
  char	font[100];
  ix = (int)( ((x - win_x_min) * win_idx) * win_sx );
  iy = (int)( win_sy - ((y - win_y_min) * win_idy) * win_sy );

  /* font A italic */
  if( strcmp( string2, "A0")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--8-80-75-75-p-50-iso8859-1"); }
  else if( strcmp( string2, "A1")==0 ){  
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--11-80-100-100-p-60-iso8859-1"); }
  else if( strcmp( string2, "A2")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--14-100-100-100-p-82-iso8859-1"); }
  else if( strcmp( string2, "A3")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--17-120-100-100-p-92-iso8859-1"); }
  else if( strcmp( string2, "A4")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--20-140-100-100-p-103-iso8859-1"); }
  else if( strcmp( string2, "A5")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--25-180-100-100-p-138-iso8859-1"); }
  else if( strcmp( string2, "A6")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-o-normal--34-240-100-100-p-182-iso8859-1"); }

  /* font B roman */
  else if( strcmp( string2, "B0")==0 ){ 
    strcpy(font,"-adobe-helvetica-bold-r-normal--8-80-75-75-p-50-iso8859-1");}
  else if( strcmp( string2, "B1")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-r-normal--11-80-100-100-p-60-iso8859-1");}
  else if( strcmp( string2, "B2")==0 ){
    strcpy(font,
	   "-adobe-helvetica-bold-r-normal--14-100-100-100-p-82-iso8859-1");}
  else if( strcmp( string2, "B3")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-r-normal--17-120-100-100-p-92-iso8859-1");}
  else if( strcmp( string2, "B4")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-r-normal--20-140-100-100-p-105-iso8859-1");}
  else if( strcmp( string2, "B5")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-r-normal--25-180-100-100-p-138-iso8859-1");}
  else if( strcmp( string2, "B6")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-bold-r-normal--34-240-100-100-p-182-iso8859-1");}
  
  /* font C italic */
  else if( strcmp( string2, "C0")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--8-80-75-75-p-47-iso8859-1");}
  else if( strcmp( string2, "C1")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--11-80-100-100-p-57-iso8859-1");}
  else if( strcmp( string2, "C2")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--14-100-100-100-p-78-iso8859-1");}
  else if( strcmp( string2, "C3")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--17-120-100-100-p-88-iso8859-1");}
  else if( strcmp( string2, "C4")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--20-140-100-100-p-98-iso8859-1");}
  else if( strcmp( string2, "C5")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--25-180-100-100-p-130-iso8859-1");}
  else if( strcmp( string2, "C6")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-o-normal--34-240-100-100-p-176-iso8859-1");}

  /* fort D roman */
  else if( strcmp( string2, "D0")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--8-80-75-75-p-46-iso8859-1");}
  else if( strcmp( string2, "D1")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--11-80-100-100-p-56-iso8859-1");}
  else if( strcmp( string2, "D2")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--14-100-100-100-p-76-iso8859-1");}
  else if( strcmp( string2, "D3")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--17-120-100-100-p-88-iso8859-1");}
  else if( strcmp( string2, "D4")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--20-140-100-100-p-100-iso8859-1");}
  else if( strcmp( string2, "D5")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--25-180-100-100-p-130-iso8859-1");}
  else if( strcmp( string2, "D6")==0 ){ 
    strcpy(font,
	   "-adobe-helvetica-medium-r-normal--34-240-100-100-p-176-iso8859-1");}

  /* font E italic */
  else if( strcmp( string2, "E0")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--8-80-75-75-p-56-iso8859-1");}
  else if( strcmp( string2, "E1")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--11-80-100-100-p-66-iso8859-1");}
  else if( strcmp( string2, "E2")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--14-100-100-100-p-88-iso8859-1");}
  else if( strcmp( string2, "E3")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--17-120-100-100-p-99-iso8859-1");}
  else if( strcmp( string2, "E4")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--20-140-100-100-p-111-iso8859-1");}
  else if( strcmp( string2, "E5")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--25-180-100-100-p-148-iso8859-1");}
  else if( strcmp( string2, "E6")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-i-normal"
	   "--34-240-100-100-p-193-iso8859-1");}

  /* font F roman */
  else if( strcmp( string2, "F0")==0 ){
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--8-80-75-75-p-56-iso8859-1");}
  else if( strcmp( string2, "F1")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--11-80-100-100-p-66-iso8859-1");}
  else if( strcmp( string2, "F2")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--14-100-100-100-p-87-iso8859-1");}
  else if( strcmp( string2, "F3")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--17-120-100-100-p-99-iso8859-1");}
  else if( strcmp( string2, "F4")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--20-140-100-100-p-113-iso8859-1");}
  else if( strcmp( string2, "F5")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--25-180-100-100-p-149-iso8859-1");}
  else if( strcmp( string2, "F6")==0 ){ 
    strcpy(font,
	   "-adobe-new century schoolbook-bold-r-normal"
	   "--34-240-100-100-p-193-iso8859-1");
  }
  /* font G Greek symbol */
  else if( strcmp( string2, "G0")==0 ){ 
    strcpy(font,
	   "-adobe-symbol-medium-r-normal"
	   "--8-80-75-75-p-51-adobe-fontspecific");
  }else if( strcmp( string2, "G1")==0 ){ 
    strcpy(font,"-adobe-symbol-medium-r-normal"
	   "--11-80-100-100-p-61-adobe-fontspecific");
  }else if( strcmp( string2, "G2")==0 ){ 
    strcpy(font,"-adobe-symbol-medium-r-normal"
	   "--14-100-100-100-p-85-adobe-fontspecific");
  }else if( strcmp( string2, "G3")==0 ){ 
    strcpy(font,"-adobe-symbol-medium-r-normal"
	   "--17-120-100-100-p-95-adobe-fontspecific");
  }else if( strcmp( string2, "G4")==0 ){ 
    strcpy(font,"-adobe-symbol-medium-r-normal"
	   "--20-140-100-100-p-107-adobe-fontspecific");}
  else if( strcmp( string2, "G5")==0 ){ 
    strcpy(font,"-adobe-symbol-medium-r-normal"
	   "--25-180-100-100-p-142-adobe-fontspecific");}
  else if( strcmp( string2, "G6")==0 ){ 
    strcpy(font,"-adobe-symbol-medium-r-normal"
	   "--34-240-100-100-p-191-adobe-fontspecific");}
  
  /* font H italic */
  else if( strcmp( string2, "H0")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--8-80-75-75-p-47-iso8859-1");}
  else if( strcmp( string2, "H1")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--11-80-100-100-p-57-iso8859-1");}
  else if( strcmp( string2, "H2")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--14-100-100-100-p-77-iso8859-1");}
  else if( strcmp( string2, "H3")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--17-120-100-100-p-86-iso8859-1");}
  else if( strcmp( string2, "H4")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--20-140-100-100-p-98-iso8859-1");}
  else if( strcmp( string2, "H5")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--25-180-100-100-p-128-iso8859-1");}
  else if( strcmp( string2, "H6")==0 ){ 
    strcpy(font,"-adobe-times-bold-i-normal--34-240-100-100-p-170-iso8859-1");}
  
  /* font I */
  else if( strcmp( string2, "I0")==0 ){ 
    strcpy(font,"-adobe-times-medium-r-normal--8-80-75-75-p-44-iso8859-1");
  }
  else if( strcmp( string2, "I1")==0 ){
    strcpy(font,"-adobe-times-medium-r-normal"
	   "--11-80-100-100-p-54-iso8859-1");
  }
  else if( strcmp( string2, "I2")==0 ){ 
    strcpy(font,"-adobe-times-medium-r-normal"
	   "--14-100-100-100-p-74-iso8859-1");
  }
  else if( strcmp( string2, "I3")==0 ){ 
    strcpy(font,"-adobe-times-medium-r-normal"
	   "--17-120-100-100-p-84-iso8859-1");
  }else if( strcmp( string2, "I4")==0 ){ 
    strcpy(font,"-adobe-times-medium-r-normal"
	   "--20-140-100-100-p-96-iso8859-1");
  }else if( strcmp( string2, "I5")==0 ){ 
    strcpy(font,"-adobe-times-medium-r-normal"
	   "--25-180-100-100-p-125-iso8859-1");
  }else if( strcmp( string2, "I6")==0 ){ 
    strcpy(font,"-adobe-times-medium-r-normal"
	   "--34-240-100-100-p-170-iso8859-1");
  }
  else printf("There is no font!!\n");
  
  if(flag_initialized == True)
    XSetFont(X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
	     XLoadFont(X11GD[GD_act]->disp,font));

  XDrawString(X11GD[GD_act]->disp,
	      X11GD[GD_act]->pm,
	      X11GD[GD_act]->gc,
	      ix,iy,string1,strlen(string1));
}  
/************************************************************************/
/*                                     					*/
/*  void	sgl_text_ex(x,y,string1,string2)               		*/
/*	                               					*/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_text_ex(float x,float y,char string1[],char string2[])
#else
  void	sgl_text_ex(x, y, string1[], string2[])
  float x,y; char string1[],string2[];
#endif
{
  int ix, iy;

  ix = (int)( ((x - win_x_min) * win_idx) * win_sx );
  iy = (int)( win_sy - ((y - win_y_min) * win_idy) * win_sy );

  if(flag_initialized == True)
    XSetFont(X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
	     XLoadFont(X11GD[GD_act]->disp,string2));

  /* Location of Fonts : /usr/lib/X11/fonts */
  /*             XLoadFont(X11GD[GD_act]->disp,"6x10"));    */
  /*             XLoadFont(X11GD[GD_act]->disp,"clB8x14")); */

  XDrawString(X11GD[GD_act]->disp,
	      X11GD[GD_act]->pm,
	      X11GD[GD_act]->gc,
	      ix,iy,string1,5);
}
/************************************************************************/
/*                                     					*/
/*  void	sgl_paint_triangle(x0,y0,x1,y1,x2,y2)	          	*/
/*	                                                               	*/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_paint_triangle(float x0,float y0,
                           float x1,float y1,
                           float x2,float y2)
#else
  void	sgl_paint_triangle(x0,y0,x1,y1,x2,y2)
  float x0,float y0,x1,float y1,x2,float y2;
#endif
{
  int ix0, iy0;
  int ix1, iy1;
  int ix2, iy2;
  static XPoint P[]={0,0,1,0,(int)0.5,1}; /* XPoint ¤ */

  ix0 = (int)( ((x0 - win_x_min) * win_idx) * win_sx );
  iy0 = (int)( win_sy - ((y0 - win_y_min) * win_idy) * win_sy );

  ix1 = (int)( ((x1 - win_x_min) * win_idx) * win_sx );
  iy1 = (int)( win_sy - ((y1 - win_y_min) * win_idy) * win_sy );

  ix2 = (int)( ((x2 - win_x_min) * win_idx) * win_sx );
  iy2 = (int)( win_sy - ((y2 - win_y_min) * win_idy) * win_sy );

  P[0].x=ix0;
  P[0].y=iy0;
  P[1].x=ix1;
  P[1].y=iy1;
  P[2].x=ix2;
  P[2].y=iy2;

  if(flag_initialized == True)
    XFillPolygon(X11GD[GD_act]->disp,   /* display ѿ */
		 X11GD[GD_act]->pm, /* Window id */
		 X11GD[GD_act]->gc, /* GC(Graphic Context  id) */
		 P,3,               /* XPoint ¤ΤȺɸο   */
		 Convex,            /* Polygon               */
		 CoordModeOrigin);  /* ɸкɸ  */
} 
/************************************************************************/
/*                                                                      */
/*                         sgl_window2xwd                               */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__
void sgl_window2xwd(void)
#else
  void sgl_window2xwd()
#endif
{
#define  FILENAME "-out XWD/pic%.4d.xwd"     

  static int num=0; 
  int status;
  char window_id[64],filename[32];
  sprintf(window_id,"/usr/bin/X11/xwd -id 0x%x ",
	  ((Window)(X11GD[GD_act] -> pm)-1) );
  sprintf(filename,FILENAME,num++);
  strcat(window_id,filename);
  system(window_id);

#undef FILENAME
}
/************************************************************************/
/*                                                                      */
/*                         sgl_window2jpg                               */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__
void sgl_window2jpg(void)
#else
  void sgl_window2jpg()
#endif
{
#define  FILENAME "-out JPG/pic%.4d"

  static int num=0; 
  int status;
  char window_id[64],filename[32];
  char buf[256] ;
  sprintf(window_id,"/usr/bin/X11/xwd -id 0x%x ",
	  ((Window)(X11GD[GD_act] -> pm)-1) );
  sprintf(filename,"-out JPG/pic%.4d.xwd",num++);
  strcat(window_id,filename);
  system(window_id);
  sprintf(buf,"convert JPG/pic%.4d.xwd JPG/pic%.4d.jpg", num-1, num-1) ;
  system(buf);
  sprintf(buf,"rm -f JPG/pic%.4d.xwd", num-1) ;
  system(buf);

#undef FILENAME
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*                         sgl_window2ppm                               */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__
void sgl_window2ppm(void)
#else
  void sgl_window2ppm()
#endif
{
#define  FILENAME "PPM/pic%d.ppm"
  static int num=0; 
  int status;
  char window_id[64],filename[32];

  sprintf(window_id,"-w%-d",((Window)(X11GD[GD_act] -> pm)-1) );
  sprintf(filename,FILENAME,num++);
  /* For Dec Alpha machine */
  if(!fork()) execl("/usr/local/bin/xppm_raw","xppm_raw",
		    window_id,filename,0);
  /* For Others */
  /*
    if(!fork()) execl("/usr/local/bin/xppm","xppm",
    window_id,filename,0);
  */
  else        wait(&status);
#undef FILENAME
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*                      sgl_window2ppm_reverse                          */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__
void sgl_window2ppm_reverse(void)
#else
  void sgl_window2ppm_reverse()
#endif
{
#define  FILENAME "PPM/pic%d.ppm"
  static int num=9999; 
  int status;
  char window_id[64],filename[32];

  sprintf(window_id,"-w%-d",((Window)(X11GD[GD_act] -> pm)-1) );
  sprintf(filename,FILENAME,num--);
  /* For Dec Alpha machine */
  if(!fork()) execl("/usr/local/bin/xppm_raw","xppm_raw",
		    window_id,filename,0);
  /* For Others */
  /*
    if(!fork()) execl("/usr/local/bin/xppm","xppm",
    window_id,filename,0);
  */
  else        wait(&status);
#undef FILENAME
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*                              sgl_point                               */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__
void sgl_point(float x, float y)
#else
  void sgl_point(x,y)
  float	x,y;
#endif
{
  int	ix,iy;

  ix = (int)( ((x - win_x_min) * win_idx) * win_sx );
  iy = (int)( win_sy - ((y - win_y_min) * win_idy) * win_sy );

  if(flag_initialized == True)
    XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
	       X11GD[GD_act]->gc,ix,iy);
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*                          sgl_point_pixel                             */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__
void sgl_point_pixel(int ix, int iy)
#else
  void sgl_point_pixel(ix,iy)
  int	ix,iy;
#endif
{
  iy = -iy;
  if(flag_initialized == True)
    XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
	       X11GD[GD_act]->gc,ix,iy);
}
/************************************************************************/
/*                                                                      */
/*                       sgl_2D_scalar_field                            */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__ 
void sgl_2D_scalar_field(float (*W)[480]) 
#else
  void sgl_2D_scalar_field( (*W)[480]) 
  float (W*)[480]; 
#endif 
{
  int i,j,int_W,int_Black;
  unsigned 	long color_of_pixel;
  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];

  XColor cc;
  cc.red   = 0L;
  cc.green = 0L;
  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  /* Max value of cc.blue = 65535L (=2^16-1) */
  for(i=0;i<256;i++){ 

    cc.red   = 256L * i+255L;   
    cc.green = (int)(65535L * sin( M_PI*0.5*((256*i)+255)/65535 ));
    cc.blue  = 65535L;

    /*
      cc.red   = 65535*asin( (float)i/255 )*2.0/M_PI;
      cc.green = 256*i+255;
      cc.blue  = 65535L;
    */
    /* cc  ¤ */
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }

  for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
  for(i=0;i<20;i++){ color_map2[i]=color_map[0];
  color_map2[236+i]=color_map[255];
  }
  for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      int_W = (int)W[i][win_sy-1-j]  *255; 
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi

      //      int_W = ( *(W+i*win_sx+j) ) *255;
      //      int_W = W[i][j]  *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }

      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
    
  }	
}

/************************************************************************/
/*                                                                      */
/*                       sgl_2D_scalar_field2                           */
/*                       by T.Yamaue 1998/02/02                         */
/************************************************************************/
#ifdef	__STDC__ 
void sgl_2D_scalar_field2(float (*W)[480], int mode ) 
#else
  void sgl_2D_scalar_field2( (*W)[480], mode) 
  float (W*)[480]; 
int mode;
#endif 
{
  int i,j,int_W,int_Black;
  unsigned 	long color_of_pixel;
  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];

  XColor cc;
  cc.red   = 0L;
  cc.green = 0L;
  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  /* Max value of cc.blue = 65535L (=2^16-1) */
  for(i=0;i<256;i++){
    switch (mode){
    case 0:
      {
	cc.red   = 256L * i+255L;   
	cc.green = (int)(65535L * sin( M_PI*0.5*((256*i)+255)/65535 ));
	cc.blue  = 65535L;
	break ;
      }
    case 1:
      {
	if (i < 64) {
	  cc.red = 0L ;
	  cc.green = (int)(65535L * sin( M_PI*0.5*((256*4*i)+255)/65535 ));
	  cc.blue = 65535L ;
	} else if (i < 128) {
	  cc.red = 0L ;
	  cc.green = 65535L ;
	  cc.blue 
	    = (int)(65535L*(1.0-sin( M_PI*0.5*((256*4*(i-64))+255)/65535) ));
	} else if (i < 192) {
	  cc.red = (int)(65535L * sin( M_PI*0.5*((256*4*(i-128))+255)/65535));
	  cc.green = 65535L ;
	  cc.blue = 0L ;
	} else {
	  cc.red = 65535L ;
	  cc.green 
	    = (int)(65535L*(1.0-sin( M_PI*0.5*((256*4*(i-192))+255)/65535 )));
	  cc.blue = 0L ;
	}
	break ;
      }
    case 2:
      {
	if (i < 51) {
	  cc.red = (int)(65535L*(1.0-sin( M_PI*0.5*((256*5*i)+255)/65535) ) );
	  cc.green = 0L;
	  cc.blue = 65535L ;
	} else if (i < 102) {
	  cc.red = 0L ;
	  cc.green = (int)(65535L * sin( M_PI*0.5*((256*5*(i-51))+255)/65535));
	  cc.blue = 65535L ;
	} else if (i < 153) {
	  cc.red = 0L ;
	  cc.green = 65535L ;
	  cc.blue 
	    = (int)(65535L*(1.0-sin( M_PI*0.5*((256*5*(i-102))+255)/65535) ));
	} else if (i < 204) {
	  cc.red = (int)(65535L * sin( M_PI*0.5*((256*5*(i-153))+255)/65535));
	  cc.green = 65535L ;
	  cc.blue = 0L ;
	} else {
	  cc.red = 65535L ;
	  cc.green
	    =(int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-204))+255)/65535 ))) ;
	  cc.blue = 0L ;
	}
	break ;
      }
    case 3:
      {
	if (i < 51) {
	  cc.red = 0L ;
	  cc.green = (int)(65535L * sin( M_PI*0.5*((256*5*i)+255)/65535 ));
	  cc.blue = 65535L ;
	} else if (i < 102) {
	  cc.red = 0L ;
	  cc.green = 65535L ;
	  cc.blue 
	    = (int)(65535L*(1.0-sin( M_PI*0.5*((256*5*(i-51))+255)/65535) ) );
	} else if (i < 153) {
	  cc.red = (int)(65535L * sin( M_PI*0.5*((256*5*(i-102))+255)/65535));
	  cc.green = 65535L ;
	  cc.blue = 0L ;
	} else {
	  cc.red = 65535L ;
	  cc.green 
	    = (int)(65535L*(1.0-sin( M_PI*0.5*((256*2.5*(i-153))+255)/65535)));
	  cc.blue = 0L ;
	}
	break ;
      }
    default:
      {
	cc.red   = 256L * i+255L;   
	cc.green = (int)(65535L * sin( M_PI*0.5*((256*i)+255)/65535 ));
	cc.blue  = 65535L;
	break ;
      }
    }
	
    /* cc  ¤ */
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }
  
  if (mode < 1 || mode > 3) {
    for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
    for(i=0;i<20;i++){ color_map2[i]=color_map[0];
    color_map2[236+i]=color_map[255];
    }
    for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  }
  
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      int_W = (int)W[i][win_sy-1-j]  *255; 
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi
      //      int_W = W[i][j]  *255;
      //int_W = ( *(W+i*win_sy+j) ) *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }
	  
      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
    
  }	
}

#ifdef	__STDC__ 
void sgl_2D_scalar_field2(float (*W), int mode ) 
#else
  void sgl_2D_scalar_field2( (*W), mode) 
  float (W*);
int mode;
#endif 
{
  int i,j,int_W,int_Black;
  unsigned 	long color_of_pixel;
  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];

  XColor cc;
  cc.red   = 0L;
  cc.green = 0L;
  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  /* Max value of cc.blue = 65535L (=2^16-1) */
  for(i=0;i<256;i++){
    switch (mode){
    case 0:
      {
	cc.red   = 256L * i+255L;   
	cc.green = (int)(65535L * sin( M_PI*0.5*((256*i)+255)/65535 ));
	cc.blue  = 65535L;
	break ;
      }
    case 1:
      {
	if (i < 64) {
	  cc.red = 0L ;
	  cc.green = (int)(65535L * sin( M_PI*0.5*((256*4*i)+255)/65535 ));
	  cc.blue = 65535L ;
	} else if (i < 128) {
	  cc.red = 0L ;
	  cc.green = 65535L ;
	  cc.blue 
	    =(int)(65535L*(1.0-sin( M_PI*0.5*((256*4*(i-64))+255)/65535)) ) ;
	} else if (i < 192) {
	  cc.red = (int)( 65535L * sin( M_PI*0.5*((256*4*(i-128))+255)/65535));
	  cc.green = 65535L ;
	  cc.blue = 0L ;
	} else {
	  cc.red = 65535L ;
	  cc.green 
	    = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*4*(i-192))+255)/65535)));
	  cc.blue = 0L ;
	}
	break ;
      }
    case 2:
      {
	if (i < 51) {
	  cc.red = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*i)+255)/65535) ));
	  cc.green = 0L;
	  cc.blue = 65535L ;
	} else if (i < 102) {
	  cc.red = 0L ;
	  cc.green 
	    = (int)( 65535L * sin( M_PI*0.5*((256*5*(i-51))+255)/65535 ) );
	  cc.blue = 65535L ;
	} else if (i < 153) {
	  cc.red = 0L ;
	  cc.green = 65535L ;
	  cc.blue 
	    = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-102))+255)/65535)));
	} else if (i < 204) {
	  cc.red = (int)( 65535L * sin( M_PI*0.5*((256*5*(i-153))+255)/65535));
	  cc.green = 65535L ;
	  cc.blue = 0L ;
	} else {
	  cc.red = 65535L ;
	  cc.green 
	    = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-204))+255)/65535)));
	  cc.blue = 0L ;
	}
	break ;
      }
    case 3:
      {
	if (i < 51) {
	  cc.red = 0L ;
	  cc.green = (int)( 65535L * sin( M_PI*0.5*((256*5*i)+255)/65535 ) );
	  cc.blue = 65535L ;
	} else if (i < 102) {
	  cc.red = 0L ;
	  cc.green = 65535L ;
	  cc.blue 
	    = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-51))+255)/65535) ));
	} else if (i < 153) {
	  cc.red = (int)( 65535L * sin( M_PI*0.5*((256*5*(i-102))+255)/65535));
	  cc.green = 65535L ;
	  cc.blue = 0L ;
	} else {
	  cc.red = 65535L ;
	  cc.green 
	    = (int)(65535L*(1.0-sin( M_PI*0.5*((256*2.5*(i-153))+255)/65535)));
	  cc.blue = 0L ;
	}
	break ;
      }
    default:
      {
	cc.red   = 256L * i+255L;   
	cc.green = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
	cc.blue  = 65535L;
	break ;
      }
    }
	
    /* cc  ¤ */
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }
  
  if (mode < 1 || mode > 3) {
    for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
    for(i=0;i<20;i++){ color_map2[i]=color_map[0];
    color_map2[236+i]=color_map[255];
    }
    for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  }
  
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      int_W = (int)( *(W+i*win_sy+win_sy-1-j) ) *255;
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi
      //	  int_W = ( *(W+i*win_sy+j) ) *255;
      //int_W = W[i][j]  *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }
	  
      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
    
  }	
}

//
// Variable Color Scalar Field Display Routine.
//     Added by kro '98.12.4(Fri.)
//
//  color_mode:
//     0  0  1  1  1  1  1  1
//     NN NN FR FG FB TR TG TB
//        F : From , T : To , R : Red , G = Green , B : Blue
//
#ifdef	__STDC__ 
void sgl_2D_scalar_field(float (*W)[480], int color_mode) 
#else
  void sgl_2D_scalar_field( (*W)[480], color_mode)
  float (W*)[480];
int color_mode;
#endif 
{
  int i,j,int_W,int_Black;
  unsigned 	long color_of_pixel;
  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];
  unsigned      long color_red[256];
  unsigned      long color_green[256];
  unsigned      long color_blue[256];

  XColor cc;                       
  cc.red   = 0L;
  cc.green = 0L;
  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  for(i=0;i<256;i++){ 
    color_red[i]     = 0L;
    color_green[i]   = 0L;
    color_blue[i]    = 0L;
  }	

  if (color_mode & 1){
    for(i=0;i<256;i++){ 
      color_blue[i] = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    }
  }
  if(color_mode & 2){
    for(i=0;i<256;i++){ 
      color_green[i] = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    }
  }
  if(color_mode & 4){
    for(i=0;i<256;i++){ 
      color_red[i]   = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    }
  }
  if(color_mode & 8){
    for(i=0;i<256;i++){ 
      color_blue[i] = color_blue[i]
	+ (int)( 65535L * (1-sin( M_PI*0.5*((256*i)+255)/65535 )) );
    }
  }	
  if(color_mode & 16){
    for(i=0;i<256;i++){ 
      color_green[i] = color_green[i]
	+ (int)( 65535L * (1-sin( M_PI*0.5*((256*i)+255)/65535 )) );
    }
  }	
  if(color_mode & 32){
    for(i=0;i<256;i++){ 
      color_red[i]   = color_red[i]
	+ (int)( 65535L * (1-sin( M_PI*0.5*((256*i)+255)/65535 )) );
    }
  }	
  /* cc  ¤ */
  for(i=0;i<256;i++){ 
    cc.red   = color_red[i];
    cc.green = color_green[i];
    cc.blue  = color_blue[i];
		
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }	
	
  for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
  for(i=0;i<20;i++){ color_map2[i]=color_map[0];
  color_map2[236+i]=color_map[255];
  }
  for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      //int_W = ( *(W+i*win_sy+win_sy-1-j) ) *255;
      int_W = (int)W[i][win_sy-1-j]  *255;
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi
      //      int_W = ( *(W+i*win_sx+j) ) *255;
      //      int_W = W[i][j]  *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }

      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
    
  }	
}

// Variable Color Scalar Field Display Routine.
//     Added by T.Yamaue '98.12.4(Fri.)
//
//	[color1] is the color of gradation start point
//		, corresponding value = 0.0
//	[color2] is the color of gradation end point
//	        , corresponding value = 1.0
//
//	Both of [color1] and [color2] must be previously allocated by
//	the function, int	sgl_colordef(char *colname).
//
#ifdef	__STDC__ 
void sgl_2D_scalar_field(float (*W)[480], int color1, int color2)
#else
  void sgl_2D_scalar_field( (*W)[480], color1, color2)
  float (W*)[480];
int color1;
int color2;
#endif 
{
  int i,j,int_W,int_Black;

  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];
  unsigned 	long color_of_pixel;
  unsigned  long color_red[256];
  unsigned  long color_green[256];
  unsigned  long color_blue[256];
	
  XColor cc ;

  XColor cc1, cc2;

  cc1.pixel = color_pixel[color1] ;
  cc2.pixel = color_pixel[color2] ;

  cc1.blue = color_rgb[color1].blue ;
  cc1.green = color_rgb[color1].green ;
  cc1.red = color_rgb[color1].red ;

  cc2.blue = color_rgb[color2].blue ;
  cc2.green = color_rgb[color2].green ;
  cc2.red = color_rgb[color2].red ;
	
  for(i=0;i<256;i++){
    color_red[i] = (cc1.red + (cc2.red - cc1.red)*i/255) ;
    color_green[i] = (cc1.red + (cc2.green - cc1.green)*i/255);
    color_blue[i] = (cc1.blue + (cc2.blue - cc1.blue)*i/255);
  }
	
  for(i=0;i<256;i++){ 
    cc.red   = color_red[i];
    cc.green = color_green[i];
    cc.blue  = color_blue[i];
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }	
	
  for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
  for(i=0;i<20;i++){ color_map2[i]=color_map[0];
  color_map2[236+i]=color_map[255];
  }
  for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      //int_W = ( *(W+i*win_sy+win_sy-1-j) ) *255;
      int_W = (int)W[i][win_sy-1-j]  *255;
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi
      //      int_W = ( *(W+i*win_sx+j) ) *255;
      //      int_W = W[i][j]  *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }

      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
    
  }	
}

//
// Color Scalar Field Display 3 Routine for any Display Size.
//     Added by kro '98.12.10(Thr)
//   Using abobe
//       sgl_2D_scalar_field(float (*W)[480]) 
//       sgl_2D_scalar_field(float (*W)[480], int color_mode) 
//       sgl_2D_scalar_field(float (*W)[480], int color1, int color2) 
//   See How to use these Routine.
//
#ifdef	__STDC__ 
void sgl_2D_scalar_field(float (*W)) 
#else
  void sgl_2D_scalar_field( (*W)) 
  float (W*); 
#endif 
{
  int i,j,int_W,int_Black;
  unsigned 	long color_of_pixel;
  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];

  XColor cc;
  cc.red   = 0L;
  cc.green = 0L;
  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  /* Max value of cc.blue = 65535L (=2^16-1) */
  for(i=0;i<256;i++){ 

    cc.red   = 256L * i+255L;   
    cc.green = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    cc.blue  = 65535L;

    /* cc  ¤ */
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }

  for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
  for(i=0;i<20;i++){ color_map2[i]=color_map[0];
  color_map2[236+i]=color_map[255];
  }
  for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      int_W = (int)( *(W+i*win_sy+win_sy-1-j) ) *255;
      //      int_W = W[i][win_sy-1-j]  *255;
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi			
      //      int_W = ( *(W+i*win_sy+j) ) *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }
      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
  }	
}
/*--------------------------------------------------------------------
//
// Color Scalar Field Display 3 Routine for any Display Size.
//     Added by kro '98.12.10(Thr)
//   Using abobe
//       sgl_2D_scalar_field(float (*W)[480]) 
//       sgl_2D_scalar_field(float (*W)[480], int color_mode) 
//       sgl_2D_scalar_field(float (*W)[480], int color1, int color2) 
//   See How to use these Routine.
//
--------------------------------------------------------------------*/
#ifdef	__STDC__ 
void sgl_2D_scalar_field(float (*W), int color_mode) 
#else
  void sgl_2D_scalar_field( (*W), color_mode)
  float (W*);
int color_mode;
#endif 
{
  int i,j,int_W,int_Black;
  unsigned 	long color_of_pixel;
  unsigned 	long color_map[256];
  unsigned 	long color_map2[256];
  unsigned      long color_red[256];
  unsigned      long color_green[256];
  unsigned      long color_blue[256];

  XColor cc;                       
  cc.red   = 0L;
  cc.green = 0L;
  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  for(i=0;i<256;i++){ 
    color_red[i]     = 0L;
    color_green[i]   = 0L;
    color_blue[i]    = 0L;
  }	

  if (color_mode & 1){
    for(i=0;i<256;i++){ 
      color_blue[i] = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    }
  }
  if(color_mode & 2){
    for(i=0;i<256;i++){ 
      color_green[i] = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    }
  }
  if(color_mode & 4){
    for(i=0;i<256;i++){ 
      color_red[i]   = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
    }
  }
  if(color_mode & 8){
    for(i=0;i<256;i++){ 
      color_blue[i] = color_blue[i]
	+ (int)( 65535L * (1-sin( M_PI*0.5*((256*i)+255)/65535 )) );
    }
  }	
  if(color_mode & 16){
    for(i=0;i<256;i++){ 
      color_green[i] = color_green[i]
	+ (int)( 65535L * (1-sin( M_PI*0.5*((256*i)+255)/65535 )) );
    }
  }	
  if(color_mode & 32){
    for(i=0;i<256;i++){ 
      color_red[i]   = color_red[i]
	+ (int)( 65535L * (1-sin( M_PI*0.5*((256*i)+255)/65535 )) );
    }
  }	
  /* cc  ¤ */
  for(i=0;i<256;i++){ 
    cc.red   = color_red[i];
    cc.green = color_green[i];
    cc.blue  = color_blue[i];

    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }	
	
  for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
  for(i=0;i<20;i++){ color_map2[i]=color_map[0];
  color_map2[236+i]=color_map[255];
  }
  for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      int_W = (int)( *(W+i*win_sy+win_sy-1-j) ) *255;
      //      int_W = W[i][win_sy-1-j]  *255;
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi		
      //      int_W = ( *(W+i*win_sy+j) ) *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }

      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
  }	
}

//
// Color Scalar Field Display 3 Routine for any Display Size.
//     Added by kro '98.12.10(Thr)
//   Using abobe
//       sgl_2D_scalar_field(float (*W)[480]) 
//       sgl_2D_scalar_field(float (*W)[480], int color_mode) 
//       sgl_2D_scalar_field(float (*W)[480], int color1, int color2) 
//   See How to use these Routine.
//
#ifdef	__STDC__ 
void sgl_2D_scalar_field(float (*W), int color1, int color2)
#else
  void sgl_2D_scalar_field( (*W), color1, color2)
  float (W*)
  int color1;
int color2;
#endif 
{
  int i,j,int_W,int_Black;

  unsigned  long color_map[256];
  unsigned  long color_map2[256];
  unsigned  long color_of_pixel;
  unsigned  long color_red[256];
  unsigned  long color_green[256];
  unsigned  long color_blue[256];
	
  XColor cc ;

  XColor cc1, cc2;

  cc1.pixel = color_pixel[color1] ;
  cc2.pixel = color_pixel[color2] ;

  cc1.blue = color_rgb[color1].blue ;
  cc1.green = color_rgb[color1].green ;
  cc1.red = color_rgb[color1].red ;

  cc2.blue = color_rgb[color2].blue ;
  cc2.green = color_rgb[color2].green ;
  cc2.red = color_rgb[color2].red ;
	
  for(i=0;i<256;i++){
    color_red[i] = (cc1.red + (cc2.red - cc1.red)*i/255) ;
    color_green[i] = (cc1.red + (cc2.green - cc1.green)*i/255);
    color_blue[i] = (cc1.blue + (cc2.blue - cc1.blue)*i/255);
  }
	
  for(i=0;i<256;i++){ 
    cc.red   = color_red[i];
    cc.green = color_green[i];
    cc.blue  = color_blue[i];
    XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
    color_map[i]=cc.pixel;
  }	
	
  for(i=0;i<216;i++){ color_map2[i+20]=color_map[i]; }
  for(i=0;i<20;i++){ color_map2[i]=color_map[0];
  color_map2[236+i]=color_map[255];
  }
  for(i=0;i<256;i++){ color_map[255-i]=color_map2[i]; }
  for(i=0;i<win_sx;i++){ 
    for(j=0;j<win_sy;j++){ 
      int_W = (int)( *(W+i*win_sy+win_sy-1-j) ) *255;
      //      int_W = W[i][win_sy-1-j]  *255;
      //yդޤ褵Τ 1999.3.16 by T.Taniguchi
      //      int_W = ( *(W+i*win_sy+j) ) *255;
      if( int_W >= 0 && int_W < 256 ){
	color_of_pixel=color_map[int_W];
      }else if( int_W > 255 ){
	color_of_pixel=int_Black; 
      }else{
	color_of_pixel=color_map[0];
      }

      XSetForeground(X11GD[GD_act]->disp, 
		     X11GD[GD_act]->gc,color_of_pixel);

      XDrawPoint(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc,i,j);

    }/* printf("pixel=%d i=%d int_W= %d\n",color_of_pixel,i,int_W); */
  }	
}

/************************************************************************/
/*                                                                      */
/*                       sgl_2D_scalar_field_to_PS                      */ 
/*                       by T.Taniguchi 1999/04/20                      */
/*                                                                      */
/* void sgl_2D_scalar_field_to_PS( float      (*W),                     */
/*                                 char*      name,                     */
/*			           int        use_input_min_max=0,      */
/*			           double     input_min=0.0,            */
/*                                 double     input_max=1.0       )     */
/*                                                                      */
/*        W[][]             :  two dimensional scalar field             */
/*        name              :  name of a Postscript File                */
/*	  use_input_min_max :  0: no use, 1: use                        */
/*        input_min         :  minimum value of the grey scale          */
/*        input_max         :  maximum value of the grey scale          */
/*                                                                      */
/************************************************************************/
#ifdef	__STDC__ 
void sgl_2D_scalar_field_to_PS( float (*W), char* name, 
																int    use_input_min_max,
																double input_min, 
																double input_max    ) 
#else
void sgl_2D_scalar_field_to_PS( ) 
float  (*W);
char*  name;
int    use_input_min_max=0,
double min=0.0, max=1.0;
#endif 
{

  double XOFFSET			= 0.0;
  double YOFFSET			= 0.0;
  double PAGE_HEIGHT	= 841.9;
  double PAGE_WIDTH		= 595.3;
  double MAX_DIMENSION= 500.0;
  double TFSIZE	      = 32.0;
  double AFSIZE	      = 24.0;
  double MFSIZE	      = 21.0;

  char *fname,*pt;
  int i,j,tmp, count,c;
  int nx,ny;
  int rnx,rny;
  double max,min,temp, cm2pt;
  double max0 = 0,min0 = 0;
  double x0,dx,y0,dy;
  int bx0,bx1,by0,by1;
  double lx0,lx1,ly0,ly1;
  double rlx0,rlx1,rly0,rly1;
  
  /*	Default Parameter	*/
  char *ttl = NULL, *xax = NULL, *yax = NULL;
  char *tfont,*afont,*mfont;
  double tfsize = TFSIZE,afsize = AFSIZE,mfsize = MFSIZE;
  int ngeo = 0;
  int error = 0;
  int epsf = 0;
  int landscape = 0;
  int bounding_box = 0;
  int measure = 1;
  int usefont;
  int cm = 1;		/*	Default Unit is cm	*/
  int ori = 0; 
  int wb_flag = 0;
  double lw = 2.0;      /* Line Width of Boundary Box */
  double page_width  = PAGE_WIDTH  - 2.0 * XOFFSET;
  double page_height = PAGE_HEIGHT - 2.0 * YOFFSET;
  double d0 = 0.0,d1 = 1.0;
  double v_white = 0.0,v_black =1.0;
  FILE *fp = fopen( name, "w" );

  if( use_input_min_max !=0 && use_input_min_max !=1 ){
    fprintf(stderr,"In function sgl_2D_scalar_field_to_PS() \n"
	    "value of 2nd parameter "
	    "\"use_input_min_max\" is wrong\n" 
	    "It should be 0: no use or 1: use\n" ); 
  }

  /*	Initialize Parameter	*/
  tfont = "Times-Bold";
  afont = "Times-Roman";
  mfont = "Times-Italic";

  fname = NULL;

  dx=5.0;
  dy=5.0;
  if( win_sx > win_sy ){ 
    dy *= (double)win_sy/win_sx; 
  }else if( win_sy > win_sx ){ 
    dx *= (double)win_sx/win_sy; 
  }

  x0=0.0;  y0=0.0;  // Offset Value

  //  epsf = 1;     //
  bounding_box = 1; // Attach Bounding Box
  measure = 0;      // 
  cm = 1;           // Unit of Length is 1: centimeter 0: PT (point)

  if( use_input_min_max == 1 ){
    wb_flag = 1;
    v_white = input_max;
    v_black = input_min;
  }else{
    // use_input_min_max == 0
    wb_flag = 0;
  }

  /*	Translate CM to PT	*/
  if (cm == 1) {
    cm2pt = 72.0 / 2.54;
    dx *= cm2pt;
    dy *= cm2pt;
    x0 *= cm2pt;
    y0 *= cm2pt;
  }
  
  rnx = win_sx;
  rny = win_sy;
  rlx0 = 0.0;
  rly0 = 0.0;
  rlx1 = rnx;
  rly1 = rny;
  lx1 = rnx;
  ly1 = rny;
  nx = rnx;
  ny = rny;
  /*	Read Raw Data	, calculation of min and max */
  max0 = min0 = (double)W[0];
  for (j = 0; j < win_sy; j++) {
    for (i = 0; i < win_sx; i++) {
      temp = W[j*win_sx+i];
      if( temp > max0 ) max0 = temp;
      if( temp < min0 ) min0 = temp;
    }
  }
  max=max0; min=min0;
  /*	Set Bounding Box	*/
  bx0 = (int) floor(x0-lw/2.0),	bx1 = (int) ceil(x0+dx+lw/2.0);
  by0 = (int) floor(y0-lw/2.0),	by1 = (int) ceil(y0+dy+lw/2.0);
  
  /*	Output of PostScript	*/
  fprintf( fp, "%%!PS-Adobe-3.0");
  if (epsf == 1)
    fprintf( fp, " EPSF-3.0\n");
  else
    fprintf( fp, "\n");
  fprintf( fp, "%%%%Creator: shading in SGL "
	   "(Create Shading Image from Lattice Data)\n");
  fprintf( fp, "%%%%Title: (%s)\n",name );
  if ((pt = getenv("USER")) != NULL) {
    fprintf( fp, "%%%%For: (%s",pt);
    if ((pt = getenv("HOST")) != NULL)
      fprintf( fp, "@%s)\n",pt);
    else
      fprintf( fp, ")\n");
  }
  fprintf( fp, "%%%%BoundingBox: %d %d %d %d\n",bx0,by0,bx1,by1);
  
  /*	Font Resources	*/
  if (usefont == 1) {
    fprintf( fp, "%%%%RequiredResources:");
    if (ttl != NULL)
      fprintf( fp, " %s",tfont);
    if (xax != NULL || yax != NULL)
      fprintf( fp, " %s",afont);
    if (measure == 1)
      fprintf( fp, " %s",mfont);
    fprintf( fp, "\n");
  }
  fprintf( fp, "%%%%EndComments\n\n");
  
  fprintf( fp, "%%%%BeginSetting\n");
  fprintf( fp, "%%Define functions\n");
  if (usefont == 1) {
    fprintf( fp, "/Font {findfont exch scalefont setfont} bind def\n");
    fprintf( fp, "/CXJ {exch 2 index stringwidth pop 2 div sub exch} def\n");
    fprintf( fp, "/RXJ {exch 2 index stringwidth pop sub exch} def\n");
  }
  fprintf( fp, "%%%%EndSetting\n\n");
  
  /*	Begin Body	*/
  fprintf( fp, "save\n");
  fprintf( fp, "/picstr %d string def\n", nx);
  
  /*	Rotation (if required)	*/
  fprintf( fp, "%g %g translate\n",XOFFSET + x0,YOFFSET + y0);
  fprintf( fp, "%g %g scale\n", dx, dy);
  fprintf( fp, "%d %d 8 [%d 0 0 %d 0 0]\n", nx, ny, nx, ny);
  fprintf( fp, "{currentfile picstr readhexstring pop}\n");
  fprintf( fp, "%%%%BeginData: %d Hex Lines\n",(nx * ny - 1) / 32 + 1);
  fprintf( fp, "image\n");
  for (j = 0 , count = 1; j < win_sy; j++) {
    for (i = 0; i < win_sx; i++,count++) {
      if (wb_flag == 0){
	tmp = 255-(int)floor((W[j+i*win_sy]-v_white)
			     /(v_black-v_white)*256.0);
      }else{
	tmp = 255-(int)floor((W[j+i*win_sy]-min)/(max-min)*256.0);
      }
      if (tmp < 0){ tmp = 0; }else if(tmp > 255 ){ tmp = 255; }

      fprintf(fp,"%02x",tmp);

      if (count > 31) {
	fprintf( fp, "\n");
	count = 0;
      }
    }
  }
  fprintf( fp, "\n");
  fprintf( fp, "%%%%EndData\n\n");
  
  /*	Scale Back to POINT unit	*/
  if (usefont == 1 || bounding_box == 1) {
    fprintf( fp, "%% Scale Back to POINT unit\n");
    fprintf( fp, "%g %g scale\n\n",1.0/dx,1.0/dy);
  }
  
  /*	Set LineWidth	*/
  if (measure == 1 || bounding_box == 1) {
    fprintf( fp, "%% Set Line Style\n");
    fprintf( fp, "%g setlinewidth [] 0 setdash\n\n",lw);
  }
  
  /*	Measure	*/
  if (measure == 1) {
    fprintf( fp, "%% Load Font\n");
    fprintf( fp, "%g /%s Font\n\n",mfsize,mfont);
    
    fprintf( fp, "%% Draw X-Measure\n");
    fprintf( fp, "%g 0 moveto\n",-lw*2.0);
    fprintf( fp, "%g 0 lineto\n",dx);
    fprintf( fp, "%g %g lineto\n",dx,-lw*2.0);
    fprintf( fp, "stroke\n");
    fprintf( fp, "(%g) %g %g CXJ moveto show\n",lx0,0.0,-lw*2.0-mfsize);
    fprintf( fp, "(%g) %g %g CXJ moveto show\n\n",lx1,dx,-lw*2.0-mfsize);
    
    fprintf( fp, "%% Draw Y-Measure\n");
    fprintf( fp, "0 %g moveto\n",-lw*2.0);
    fprintf( fp, "0 %g lineto\n",dy);
    fprintf( fp, "%g %g lineto\n",-lw*2.0,dy);
    fprintf( fp, "stroke\n");
    fprintf( fp, "(%g) %g %g RXJ moveto show\n",
	     ly0,-lw*2.0-mfsize/2.0,-mfsize/3.0);
    fprintf( fp, "(%g) %g %g RXJ moveto show\n\n",
	     ly1,-lw*2.0-mfsize/2.0,dy-mfsize/3.0);
  }
  
  /* Bounding Box (if required)	*/
  if (bounding_box == 1) {
    fprintf( fp, "%% Draw Bounding Box\n");
    fprintf( fp, "0 0 moveto\n");
    fprintf( fp, "%g 0 lineto\n",dx);
    fprintf( fp, "%g %g lineto\n",dx,dy);
    fprintf( fp, "0 %g lineto\n",dy);
    fprintf( fp, "0 0 lineto\n");
    fprintf( fp, "stroke\n\n");
  }
  
  /*	If '-eps' option is used, 'showpage' is commented out	*/
  if (epsf == 1){
    fprintf( fp, "%%");
  }else{
    fprintf( fp, "showpage\n");
    fprintf( fp, "restore\n\n");
    fprintf( fp, "%%%%Trailer\n");
  }
  
  /* output profile of data to stderr */
  fprintf(stderr,"%%    AXIS     : NUMBER   MINIMUM   MAXIMUM\n");
  fprintf(stderr,"%% horizontal  : %6d  %8.3lf  %8.3lf\n",nx,lx0,lx1);
  fprintf(stderr,"%% vertical    : %6d  %8.3lf  %8.3lf\n",ny,ly0,ly1);
  fprintf(stderr,"%% gray        : ------  %8.3lf  %8.3lf\n",min0,max0);
  if (wb_flag == 0)	
    fprintf(stderr,"%%       range :  [white(%8.3lf)(%8.3lf)black]\n",\
	    min,max);
  else
    fprintf(stderr,"%%       range :  [white(%8.3lf)(%8.3lf)black]\n",\
	    v_white,v_black);
  
}
/************************************************************************/
/*                                                            					*/
/*  void	sgl_paint_triangle2(x0,y0,x1,y1,x2,y2,color,mode)	            */
/*              color : float [0,1]                                     */
/*	            mode : int [0,3]                                        */
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_paint_triangle2(float x0,float y0,
													float x1,float y1,
													float x2,float y2, 
													float color, int mode )
#else
void	sgl_paint_triangle2(x0,y0,x1,y1,x2,y2, color, mode)
float x0,float y0,x1,float y1,x2,float y2;
float color;
int mode;
#endif
{
  int i;
  int int_Black;
  int ix0, iy0;
  int ix1, iy1;
  int ix2, iy2;
  static XPoint P[]={0, 0, 1, 0, (int)0.5, 1}; /* XPoint ¤ */
  unsigned long color_of_pixel;

  ix0 = (int)( ((x0 - win_x_min) * win_idx) * win_sx );
  iy0 = (int)( win_sy - ((y0 - win_y_min) * win_idy) * win_sy );

  ix1 = (int)( ((x1 - win_x_min) * win_idx) * win_sx );
  iy1 = (int)( win_sy - ((y1 - win_y_min) * win_idy) * win_sy );

  ix2 = (int)( ((x2 - win_x_min) * win_idx) * win_sx );
  iy2 = (int)( win_sy - ((y2 - win_y_min) * win_idy) * win_sy );

  P[0].x=ix0;	P[0].y=iy0;
  P[1].x=ix1;	P[1].y=iy1;
  P[2].x=ix2;	P[2].y=iy2;

  XColor cc;                       
  cc.red   = 0L;  cc.green = 0L;  cc.blue  = 0L;
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc);
  int_Black = cc.pixel;

  switch (mode){
  case 0:
    {
      i = 256 - (int)( 256*color );
      if(i<20){ i=0; }else if( i>=256 ){ i=255; }
      cc.red   = 256L * i+255L;   
      cc.green = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
      cc.blue  = 65535L;
      break ;
    }
  case 1:
    {
      i = (int)( 256*color );
      if (i < 64) {
	cc.red = 0L ;
	cc.green = (int)( 65535L * sin( M_PI*0.5*((256*4*i)+255)/65535 ) );
	cc.blue = 65535L ;
      } else if (i < 128) {
	cc.red = 0L ;
	cc.green = 65535L ;
	cc.blue 
	  = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*4*(i-64))+255)/65535) ) );
      } else if (i < 192) {
	cc.red = (int)( 65535L * sin( M_PI*0.5*((256*4*(i-128))+255)/65535) );
	cc.green = 65535L ;
	cc.blue = 0L ;
      } else {
	cc.red = 65535L ;
	cc.green 
	  = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*4*(i-192))+255)/65535 )) );
	cc.blue = 0L ;
      }
      break ;
    }
  case 2:
    {
      i = (int)( 256*color );
      if (i < 51) {
	cc.red = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*i)+255)/65535) ) );
	cc.green = 0L;
	cc.blue = 65535L ;
      } else if (i < 102) {
	cc.red = 0L ;
	cc.green = (int)( 65535L * sin( M_PI*0.5*((256*5*(i-51))+255)/65535 ));
	cc.blue = 65535L ;
      } else if (i < 153) {
	cc.red = 0L ;
	cc.green = 65535L ;
	cc.blue 
	  = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-102))+255)/65535) ) );
      } else if (i < 204) {
	cc.red = (int)( 65535L * sin( M_PI*0.5*((256*5*(i-153))+255)/65535) );
	cc.green = 65535L ;
	cc.blue = 0L ;
      } else {
	cc.red = 65535L ;
	cc.green 
	  = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-204))+255)/65535 )) );
	cc.blue = 0L ;
      }
      break ;
    }
  case 3:
    {
      i = (int)( 256*color );
      if (i < 51) {
	cc.red = 0L ;
	cc.green = (int)( 65535L * sin( M_PI*0.5*((256*5*i)+255)/65535 ) );
	cc.blue = 65535L ;
      } else if (i < 102) {
	cc.red = 0L ;
	cc.green = 65535L ;
	cc.blue 
	  = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*5*(i-51))+255)/65535) ) );
      } else if (i < 153) {
	cc.red = (int)( 65535L * sin( M_PI*0.5*((256*5*(i-102))+255)/65535) );
	cc.green = 65535L ;
	cc.blue = 0L ;
      } else {
	cc.red = 65535L ;
	cc.green 
	  = (int)( 65535L*(1.0-sin( M_PI*0.5*((256*2.5*(i-153))+255)/65535) ));
	cc.blue = 0L ;
      }
      break ;
    }
  default:
    {
      i = 256 - (int)( 256*color );
      if(i<20){ i=0; }else if( i>=256 ){ i=255; }
      cc.red   = 256L * i+255L;   
      cc.green = (int)( 65535L * sin( M_PI*0.5*((256*i)+255)/65535 ) );
      cc.blue  = 65535L;
      break ;
    }
  }
  
  /* cc  ¤ */
  XAllocColor(X11GD[GD_act]->disp,X11GD[GD_act]->colmap,&cc); 
  color_of_pixel = cc.pixel;
  
  /* Setting of ForeGround Color */
  XSetForeground(X11GD[GD_act]->disp, 
		 X11GD[GD_act]->gc,color_of_pixel);

  if(flag_initialized == True)
    XFillPolygon(X11GD[GD_act]->disp,   /* display ѿ */
		 X11GD[GD_act]->pm, /* Window id */
		 X11GD[GD_act]->gc, /* GC(Graphic Context  id) */
		 P,3,               /* XPoint ¤ΤȺɸο   */
		 Convex,            /* Polygon               */
		 CoordModeOrigin);  /* ɸкɸ  */

} 
/************************************************************************/
/* Modifiers */
/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/
/*									*/
/*  void	sgl_winsize(sx,sy)					*/
/*		redefine window size in pixels				*/
/*									*/
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_winsize(int new_sx, int new_sy)
#else
  void	sgl_winsize(new_sx,new_sy)
  int	new_sx,new_sy;
#endif
{
  if(flag_initialized != True){ /* not initialized	*/
    win_sx = new_sx;
    win_sy = new_sy;
  }
}
//
// Window Size Return Routine.
//     Added by kro '98.12.11(Fri.)
//         for using sgl_2D_scolor_field ()
//
//   int* sgl_winsize() de win_sx,win_sy wo return.
//
int* sgl_winsize()
{
  int* win_size = (int *)calloc(2,sizeof(int));
  win_size[0] = win_sx;
  win_size[1] = win_sy;
  return(win_size);
}
/************************************************************************/
/************************************************************************/
/* void	sgl_viewport(xmin,ymin,dx,dy) */
/* redefine viewport coordinate system */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_viewport(float xmin, float ymin, float dx, float dy)
#else
  void	sgl_viewport(xmin,ymin,dx,dy)
  float	xmin,ymin,dx,dy;
#endif
{ 
  int	flag,new_sx,new_sy;

  if(fabs((win_idy/win_idx) - (dx/dy)) > 1E-3)
    flag = 1;				/* window aspect ratio changed	*/ 
  else
    flag = 0;
  win_x_min = xmin;
  win_y_min = ymin;
  win_x_max = xmin + dx;
  win_y_max = ymin + dy;
  win_idx = 1.0 / dx;
  win_idy = 1.0 / dy;
  new_sx = win_sx;
  new_sy = (int)( win_sx * (dy/dx) );
  if(flag != 0)
    sgl_winsize(new_sx,new_sy);
}
/************************************************************************/
/************************************************************************/
/* void sgl_display(void)                                               */
/* display background drawings (double buffer)                          */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__
void		sgl_display(void)
#else
  void		sgl_display()
#endif
{
  XCopyArea(X11GD[GD_act]->disp,X11GD[GD_act]->pm,
	    X11GD[GD_act]->win,X11GD[GD_act]->gc,
	    0,0,win_sx,win_sy,0,0);
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc,
		  color_pixel[color_background]);
  XFillRectangle(X11GD[GD_act]->disp, X11GD[GD_act]->pm,
		 X11GD[GD_act]->gc, 0,0,win_sx,win_sy);
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc,
		  color_pixel[color_foreground]);
  XFlush(X11GD[GD_act]->disp);
  XSync(X11GD[GD_act]->disp, X11GD[GD_act]->win);
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_pos_win(int x, int y)                               */
/*              position a created window on screen                     */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_pos_win(int x, int y)
#else
  void    sgl_pos_win(x,y)
  int	x,y;
#endif
{
  XMoveWindow(X11GD[GD_act]->disp, X11GD[GD_act]->win,x,y);
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_select_win(int win)                                 */
/*              select a created X11 window for drawing                 */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_select_win(int win)
#else
  void    sgl_select_win(win)
  int	win;
#endif
{
  if(win < 0 || win >= GD_num)
    { fprintf(stderr,"SGL_SELECT_WIN: Selected window (%d) out of range\n",win);
    return;
    }
  else
    { GD_act = win;
    }
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_map_win(int win)                                    */
/*              map a created window onto screen                        */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_map_win(int win)
#else
  void    sgl_map_win(win)
  int	win;
#endif
{
  if(win < 0 || win >= GD_num)
    { fprintf(stderr,"SGL_MAP_WIN: Selected window (%d) out of range\n",win);
    return;
    }
  else
    { XMapWindow(X11GD[win]->disp, X11GD[win]->win);
    }
}

/************************************************************************/
/************************************************************************/
/*                                                                      */
/*  void        sgl_unmap_win(int win)                                  */
/*              unmap a created window                                  */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef	__STDC__
void    sgl_unmap_win(int win)
#else
  void    sgl_unmap_win(win)
  int	win;
#endif
{
  if(win < 0 || win >= GD_num)
    { fprintf(stderr,"SGL_UNMAP_WIN: Selected window (%d) out of range\n",win);
    return;
    }
  else
    { XUnmapWindow(X11GD[win]->disp, X11GD[win]->win);
    }
}
/************************************************************************/
/************************************************************************/
/* COLOR FUNCTIONS */
/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/
/* void	sgl_create_colmap(void)	*/
/* create (pseudo) color colormap */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
static void	sgl_create_colmap(void)
#else
  static void	sgl_create_colmap()
#endif
{
  XColor	exactColor,actualColor;
  int		ret;

  X11GD[GD_act]->colmap = 0;
  /***** Use default colormap if possible *****/
  if(X11GD[GD_act]->vis == DefaultVisualOfScreen( X11GD[GD_act]->scrn ) ){
    X11GD[GD_act]->colmap = DefaultColormapOfScreen( X11GD[GD_act]->scrn );
  }else{ 
    X11GD[GD_act]->colmap = XCreateColormap(X11GD[GD_act]->disp, 
					    RootWindow(X11GD[GD_act]
						       ->disp,X11GD[GD_act]
						       ->scrnn),
					    X11GD[GD_act]->vis, AllocNone);
  }
  if(X11GD[GD_act]->colmap == 0) 
    FatalError("SGL_CREATE_COLMAP: Can't create color map for visual.\n");

  ret = XAllocNamedColor(X11GD[GD_act]->disp, X11GD[GD_act]->colmap, "black",
			 &actualColor,&exactColor);
  if(ret == 0)
    fprintf(stderr,"sgl_create_colmap: Can't allocate 'black' color.\n");
  color_pixel[0] = actualColor.pixel;

  // Added by T.Yamaue 98/12/04
  color_rgb[0].blue = actualColor.blue;
  color_rgb[0].red = actualColor.red;
  color_rgb[0].green = actualColor.green;

  ret = XAllocNamedColor(X11GD[GD_act]->disp, X11GD[GD_act]->colmap, "white",
			 &actualColor,&exactColor);
  if(ret == 0)
    fprintf(stderr,"sgl_create_colmap: Can't allocate 'white' color.\n");
  color_pixel[1] = actualColor.pixel;

  // Added by T.Yamaue 98/12/04
  color_rgb[1].blue = actualColor.blue;
  color_rgb[1].red = actualColor.red;
  color_rgb[1].green = actualColor.green;

  color_stack_top = 2;
}

/************************************************************************/
/************************************************************************/
/* void	sgl_color(int)                                                  */
/* choose actual drawing color                                          */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_color(int colindex)
#else
  void	sgl_color(colindex)
  int	colindex;
#endif
{
  switch(X11GD[GD_act]->vistype)
    { case VISTYPE_GrayScale:
    case VISTYPE_StaticGray:
    case VISTYPE_PseudoColor:
    case VISTYPE_TrueColor:
	if(colindex < 0)
	  fprintf(stderr,"SGL_COLOR: There are no negativ (%d) colors.\n",
		  colindex);
	else if(colindex >= color_stack_top)
	  fprintf(stderr,"SGL_COLOR: color %d not defined.\n",colindex);
	else
	  { XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc,
			    color_pixel[colindex]);
	  color_foreground = colindex;
	  }
	break;
    }
}
/************************************************************************/
/************************************************************************/
/* void	sgl_color_background(int) */
/* choose actual background color */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_color_background(int colindex)
#else
  void	sgl_color_background(colindex)
  int	colindex;
#endif
{
  switch(X11GD[GD_act]->vistype)
    { case VISTYPE_GrayScale:
    case VISTYPE_StaticGray:
    case VISTYPE_PseudoColor:
    case VISTYPE_TrueColor:
	if(colindex < 0)
	  fprintf(stderr,"SGL_COLOR_BACKGROUND: No negativ (%d) colors.\n",
		  colindex);
	else if(colindex >= color_stack_top)
	  fprintf(stderr,"SGL_COLOR_BACKGROUND: color %d not defined.\n",
		  colindex);
	else
	  { XSetBackground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
			    color_pixel[colindex]);
	  color_background = colindex;
	  }
	break;
    }
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
		  color_pixel[color_background]);
  XFillRectangle(X11GD[GD_act]->disp, X11GD[GD_act]->pm, X11GD[GD_act]->gc,
		 0,0,win_sx,win_sy);
  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
		  color_pixel[color_foreground]);
}

/************************************************************************/
/************************************************************************/
/* void	sgl_colordef(int, char *) */
/* define a colormap entry */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
int	sgl_colordef(char *colname)
#else
  int	sgl_colordef(colname)
  char	*colname;
#endif
{
  XColor	exactColor,actualColor;
  int		ret,return_value;

  return_value = -1;
  switch(X11GD[GD_act]->vistype)
    { case VISTYPE_GrayScale:
	return_value = 1;
	break;
    case VISTYPE_StaticGray:
      return_value = 1;
      break;
    case VISTYPE_PseudoColor:
    case VISTYPE_TrueColor:
      ret = XAllocNamedColor( X11GD[GD_act]->disp, X11GD[GD_act]->colmap,
			      colname,
			      &actualColor,&exactColor );
      if(ret == 0)
	{ fprintf(stderr,"SGL_COLORDEF: Can't allocate color (%s).\n",
		  colname);
	return(-1);
	}
      else
	{ 
	  color_pixel[color_stack_top] = actualColor.pixel;
		 
	  // Added by T.Yamaue 98/12/04
	  color_rgb[color_stack_top].blue = actualColor.blue;
	  color_rgb[color_stack_top].red = actualColor.red;
	  color_rgb[color_stack_top].green = actualColor.green;

	  XSetForeground( X11GD[GD_act]->disp, X11GD[GD_act]->gc, 
			  color_pixel[color_stack_top] );
	  color_foreground = color_stack_top;
	  if(color_stack_top >= COLOR_MAX)
	    { fprintf(stderr,"SGL_COLORDEF: %d colors available only.\n",
		      COLOR_MAX);
	    return(-1);
	    }
	  else
	    { color_stack_top++;
	    return_value = (color_stack_top-1);
	    }
	}
      break;
    }
  return(return_value);
}
/************************************************************************/
/************************************************************************/
/* MISC. ...                                                            */
/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/
/* void	sgl_debugging(int)                                              */
/* set flag, print debugging information or not	                        */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	sgl_debugging(int value)
#else
  void	sgl_debugging(value)
  int	value;
#endif
{
  if(value == 0)	debugging = False;
  else			debugging = True;
}
/************************************************************************/
/************************************************************************/
/*                                                                      */
/* void	FatalError(char *)                                              */
/* printf fatal error message and exit                                  */
/*                                                                      */
/************************************************************************/
/************************************************************************/
#ifdef  __STDC__ 
void	FatalError(char *str)
#else
  void	FatalError(str)
  char	*str;
#endif
{
  fprintf(stderr,"%s",str);
  exit(-1);
}
/************************************************************************/
