/************************************************************************/
/*		                                                                  */
/*                      sgl+.h   ver.14.0                               */
/*		                                                                  */
/************************************************************************/
/*		                                                                  */
/*                      + Takashi Taniguchi                             */
/*		                                                                  */
/************************************************************************/
/*                                                                      */
/*	SGL.H		(C) 9/94 Ch. Muenkel				                                */
/*	Include file for libsgl.a library				                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*	Simple Graphics Library						                                  */
/*	An easy to use interface to the X11 window system		                */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*	Author:	Christian Muenkel					                                  */
/*		Present address:					                                        */
/*		muenkel@carbo.tphys.uni-heidelberg.de	                            */
/*		Christian Muenkel	                                                */
/*		Institut fuer Theoretische Physik                                 */
/*		Philosophenweg 19					                                        */
/*		69120 Heidelberg - Germany                                        */
/*                                                                      */
/************************************************************************/
/* version      date            comment                                 */
/*                                                                      */
/* V1.0		 1/94		first implementation 			                            */
/*		 	   9/94		final tests			                                      */
/*                                                                      */
/************************************************************************/

#ifndef		__SGL_INCLUDED
#define		__SGL_INCLUDED

#include        <stdio.h>
#include        <stdlib.h>
#include        <math.h>

#include        <string.h>             /* added by Taniguchi */
#include        <unistd.h>             /* added by Taniguchi */
#include        <sys/wait.h>           /* added by Taniguchi */

#include        <X11/Xlib.h>
#include        <X11/Xutil.h>


/************************************************************************/
/* global defines, which should be accesible from all modules		        */
/************************************************************************/
#define	VISTYPE_StaticGray		1
#define	VISTYPE_StaticColor		2
#define	VISTYPE_TrueColor		3
#define	VISTYPE_GrayScale		4
#define	VISTYPE_PseudoColor		5
#define	VISTYPE_DirectColor		6

#define	SGL_BLACK	0
#define	SGL_WHITE	1

/************************************************************************/
/* function declaration                                                 */
/************************************************************************/
#ifdef		__STDC__				/* ANSI C	*/
void		sgl_init(void);
void		sgl_initm(int);

void		sgl_viewport(float,float,float,float);
void		sgl_winsize(int,int);
void		sgl_display(void);
void		sgl_debugging(int);

int			sgl_colordef(char *);
void		sgl_color(int);
void		sgl_color_background(int);

void		sgl_line(float,float,float,float);
void		sgl_circle(float,float,float);
void		sgl_disc(float,float,float);
void		sgl_rectangle(float,float,float,float);
void		sgl_rectarea(float,float,float,float);

/* added by Taniguchi */
void    sgl_arrow(float x1, float y1, float vx, float vy, 
									float size = 0.2, int flag = 0 );
void    sgl_text(float, float, char*);
void    sgl_text2(float, float, char*, char*);
void		sgl_text_ex(float,float,char* ,char*);
void		sgl_paint_triangle(float,float,float,float,float,float);
void		sgl_window2xwd(void);
void		sgl_window2ppm(void);
void		sgl_window2ppm_reverse(void);
void		sgl_point(float, float);
void		sgl_point_pixel(int, int);
void		sgl_2D_scalar_field( float (*W)[480] );
void    sgl_paint_triangle2( float x0,float y0,
														 float x1,float y1,
														 float x2,float y2,
														 float color, 
														 int mode=0 );
/*void       ppmfile(void); */
/* addded by A.Kuroda (kro) */ 
int*		sgl_winsize();
void		sgl_2D_scalar_field(float (*W)[480], int color_mode);
void		sgl_2D_scalar_field(float (*W)); 
void		sgl_2D_scalar_field(float (*W), int color_mode); 
void		sgl_2D_scalar_field(float (*W), int color1, int color2); 
/* added by T.Yamaue */
void		sgl_2D_scalar_field(float (*W)[480], int color1, int color2);
void		sgl_2D_scalar_field2(float (*W), int mode = 0);
void		sgl_2D_scalar_field2(float (*W)[480], int mode = 0);
void		sgl_window2jpg(void);
/* added by T. Taniguch */
void		sgl_2D_scalar_field_to_PS(float (*W), char* name, 
					  int    use_input_max_min=0,
					  double min=0.0e0, double max=1.0e0 );

void		sgl_select_win(int);
void		sgl_map_win(int);
void		sgl_unmap_win(int);
void		sgl_pos_win(int,int);

void		FatalError(char *);

#else

void		sgl_init();
void		sgl_initm();

void		sgl_viewport();
void		sgl_winsize();
void		sgl_display();
void		sgl_debugging();

int	        sgl_colordef();
void		sgl_color();
void		sgl_color_background();

void		sgl_line();
void		sgl_circle();
void		sgl_disc();
void		sgl_rectangle();
void		sgl_rectarea();

/* added by Taniguchi */
void    sgl_arrow();
void		sgl_text();
void    sgl_text2();
void		sgl_text_ex();
void		sgl_paint_triangle();
void		sgl_window2xwd();
void		sgl_window2ppm();
void		sgl_window2ppm_reverse();
void		sgl_point();
void		sgl_point_pixel();
void		sgl_2D_scalar_field();
void            sgl_paint_triangle2();
/* void            ppmfile(); */

/* added by T.Yamaue */
void		sgl_2D_scalar_field2(); 
void		sgl_window2jpg();
/* added by T. Taniguch */
void		sgl_2D_scalar_field_to_PS( );

void		sgl_select_win();
void		sgl_map_win();
void		sgl_unmap_win();
void		sgl_pos_win();

void		FatalError();

#endif			/* of ifdef  __STDC__		*/

#endif			/* of ifndef __SGL_INCLUDED	*/
